/* The Second Degree v1.1 by Dan Parsons
   <dan@stepahead.net>
*/

#include <PalmOS.h>
#include "resource.h"

typedef struct {
        int x1; 
        int x2;
        int discriminant;
	int imaginary;
	char ix1[10];
	char ix2[10];
        char *notes;
} Factors;

static Err StartApplication();
static void StopApplication();
static Boolean MainFormEventHandler(EventPtr event);
static Boolean ApplicationEventHandler(EventPtr event);
static void EventLoop();
UInt32 PilotMain(UInt16 launchCode, void *cmdPBP, UInt16 launchFlags);
int QEval(int a, int b, int c);
int mysqrt(int n); // Joseph Hall <jnhall@sat.mot.com> wrote this function

Factors theFactors;

int mysqrt(int n) {
    int i;
    long k0, k1, nn;

    for (nn = i = n, k0 = 2; i > 0; i >>= 2, k0 <<= 1)
        ;
    nn <<= 2;
    for (;;) {
        k1 = (nn / k0 + k0) >> 1;
        if (((k0 ^ k1) & ~1) == 0)
            break;
        k0 = k1;
    }
    return (int) ((k1 + 1) >> 1);
}


int QEval(int a, int b, int c) {
	int a1, a2;
	theFactors.discriminant = ( (b*b) - (4*a*c) );
	if (theFactors.discriminant < 0) { 
/*		theFactors.discriminant *= -1;
		theFactors.imaginary = true; */
		return -1;
	}
	if (theFactors.imaginary) {
		a1 = ( ((-b)/(2*a)) );
		a2 = ( mysqrt( (theFactors.discriminant / (2*a) )) );
		StrPrintF(theFactors.ix1, "%i + i%i", a1, a2);
		StrPrintF(theFactors.ix2, "%i - i%i", a1, a2);
	} else {
		theFactors.x1 = ( (-b + mysqrt(theFactors.discriminant)) / (2*a));
		theFactors.x2 = ( (-b - mysqrt(theFactors.discriminant)) / (2*a));
	}
	return 0;
}


static Err StartApplication() {
	FrmGotoForm(MainForm);
	return 0;
}

static void StopApplication() {
	FrmCloseAllForms();	
}

static Boolean MainFormEventHandler(EventPtr event) {
	Boolean handled = false, result;
	FormPtr frm;
	RectangleType ResultsBox;
	FieldPtr aField;
	int a = 1, b = 1, c = 1;
	char blah[80], theResult[80], blah2[80];
	ResultsBox.topLeft.x = 2;
	ResultsBox.topLeft.y = 95;
	ResultsBox.extent.x = 156;
	ResultsBox.extent.y = 63;
	frm = FrmGetActiveForm();
	switch (event->eType) {
		case ctlSelectEvent:
			switch (event->data.ctlSelect.controlID) {
				case MainFormCompute:
					if (FldGetTextPtr(FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, MainFormAValue)))) 
						a = StrAToI(FldGetTextPtr(FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, MainFormAValue))));
					else a = 1;
					if (FldGetTextPtr(FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, MainFormBValue))))
						b = StrAToI(FldGetTextPtr(FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, MainFormBValue))));
					else b = 1;
					if (FldGetTextPtr(FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, MainFormCValue))))
						c = StrAToI(FldGetTextPtr(FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, MainFormCValue))));
					else c = 1;
					if (!a) a = 1;
					if (!b) b = 0;
					if (!c) c = 0;
					if (!QEval(a, b, c)) {
						StrPrintF(theResult, "The roots are %i and %i.                          ",theFactors.x1, theFactors.x2);
						WinDrawChars(theResult, StrLen(theResult), 4, 97);
						StrPrintF(theResult, "The discriminant is %i.                             ", theFactors.discriminant);
						WinDrawChars(theResult, StrLen(theResult), 4, 108);
						StrCopy(theResult, "                                                                            ");
						WinDrawChars(theResult, StrLen(theResult), 4, 119);
						WinDrawChars(theResult, StrLen(theResult), 4, 130);
						WinDrawChars(theResult, StrLen(theResult), 4, 141);
						WinDrawRectangleFrame(boldRoundFrame, &ResultsBox);
					} else {
						StrPrintF(theResult, "Sorry, the roots of your quadratic");
						WinDrawChars(theResult, StrLen(theResult), 4, 97);
						StrPrintF(theResult, "equation are not real and this                ");
						WinDrawChars(theResult, StrLen(theResult), 4, 108);
						StrPrintF(theResult, "program cannot comprehend such");
						WinDrawChars(theResult, StrLen(theResult), 4, 119);
						StrPrintF(theResult, "things. (d=%i)", theFactors.discriminant);
						WinDrawChars(theResult, StrLen(theResult), 4, 130);
						WinDrawRectangleFrame(boldRoundFrame, &ResultsBox);
					}
				break;
			}
		break;
		
		case frmOpenEvent:
			FrmDrawForm(FrmGetActiveForm());
			WinDrawRectangleFrame(boldRoundFrame, &ResultsBox);
			handled = true;
		break;
		
		case menuEvent:
			switch (event->data.menu.itemID) {
				case MainFormMenuHelpAbout:
					FrmCustomAlert(InfoAlert, "Coded by Dan Parsons, http://palmoslab.com/. sqrt() by Joseph Hall <jnhall@sat.mot.com>. Icon created by Janice Karin <jkarin@netway.com>. Coded in Linux using gcc, released under the GPL.", "", ""); 
				break;
				case MainFormMenuHelpInfo:
					FrmCustomAlert(InfoAlert, "I can't seem to find an atol() or atof() for PalmOS, so the numbers you enter must be integral. If a=0 or it is left blank, 1 is assumed.", "", "");
				break;
			}
			handled = true;
		break;
	}
	return handled;
}

static Boolean ApplicationEventHandler(EventPtr event) {
	FormPtr frm;
	Int16 formID;
	Boolean handled = false;
	switch (event->eType) {
		case frmLoadEvent:
			formID = event->data.frmLoad.formID;
			frm = FrmInitForm(formID);
			FrmSetActiveForm(frm);
			switch (formID) {
				case MainForm:
					FrmSetEventHandler(frm, MainFormEventHandler);
				break;
			}
			handled = true;
		break;
	}
	return handled;
}

static void EventLoop() {
	EventType event;
	UInt16 error;
	while (event.eType != appStopEvent) {
		EvtGetEvent(&event, evtWaitForever);
		if (!SysHandleEvent(&event))
			if (!MenuHandleEvent(0, &event, &error))
				if (!ApplicationEventHandler(&event))
					FrmDispatchEvent(&event);
	}
}

UInt32 PilotMain(UInt16 launchCode, void *cmdPBP, UInt16 launchFlags) {
	Err err;
	if (launchCode == sysAppLaunchCmdNormalLaunch) {
		if ((err = StartApplication()) == 0) {
			EventLoop();
			StopApplication();
		}
	}
	return err;
}
